/*整个toplevel*/
#PanelToplevel *{
/*    
     background-color: #DC143C; 
*/
}
/*窗口列表*/
/*
button {
    background-color: transparent;
    border-color: rgba(255,255,255,0.4);
    border-width: 1px;
    background-color: rgba(255, 255, 255, 0.15);
}
button:hover *{
    border-color: rgba(255,255,255,0.5);
    border-width: 1px;
    background-color: rgba(255, 255, 255, 0.3);
}
button:checked *{
    border-color: rgba(255,255,255,0.3);
    border-width: 1px;
    background-color: rgba(0,0,0,0.1);
*    background-color: red;
}
*/
#tasklist-button {
    background-color: transparent;
    border-color: rgba(255,255,255,0.4);
    border-width: 1px;
    background-color: rgba(255, 255, 255, 0.15);
}
#tasklist-button:hover {
    border-color: rgba(255,255,255,0.5);
    border-width: 1px;
    background-color: rgba(255, 255, 255, 0.3);
    background-image: image(rgba(255, 255, 255, 0.3));
}
#tasklist-button:checked {
    border-color: rgba(255,255,255,0.3);
    border-width: 1px;
    background-color: rgba(0, 0, 0, 0.1);
    background-image: image(rgba(0, 0, 0, 0.1));
}
/*
*panel的框架，没用上
#PanelFrame *{
    background-color: #DC143C;
}
*分割符，没用上
#PanelSeparator *{
    background-color: #DC143C;
}
#PanelMenu *{    
    background-color: #DC143C;
}
#UkuiPanelAppletFrameDBus *{
    background-color: #DC143C;
}
#WnckApplet *{
    background-color: #DC143C;i
}
*/
